/*
  Typical pin layout used:
   ---------------------------------------
               MFRC522      Arduino
               Reader/PCD   Uno/101
   Signal      Pin          Pin
   ---------------------------------------
   RST/Reset   RST          9
   SPI SS      SDA(SS)      10
   SPI MOSI    MOSI         11 / ICSP-4
   SPI MISO    MISO         12 / ICSP-1
   SPI SCK     SCK          13 / ICSP-3
*/

#include <SPI.h>      // Serial Peripheral Interface
#include <MFRC522.h>  // Needed for using RFID module 13.56 MHz                                                                                                                                                             

const byte NR_OF_READERS = 6;             // Numbers of RFID readers
const byte RST_PIN = 9;                   // Reset pin (default 9)
const byte ssPins[] = {2, 3, 4, 5, 6, 7}; // SS pins of all the readers
MFRC522 mfrc522[NR_OF_READERS];           // Create MFRC522 instance.
String currentIDs[NR_OF_READERS];         // Array to keep track of all the UIDs
/*
   Initialize.
*/
void setup() {

  Serial.begin(9600); // Initialize serial communications with the PC
  while (!Serial);    // Do nothing if no serial port is opened (added for Arduinos based on ATMEGA32U4)

  SPI.begin();        // Initialize Serial Peripheral Interface

  for (uint8_t reader = 0; reader < NR_OF_READERS; reader++) {
    mfrc522[reader].PCD_Init(ssPins[reader], RST_PIN);  // Init each MFRC522 card
    //    Check if the readers are recognized
    Serial.print(F("Reader "));
    Serial.print(reader);
    Serial.print(F(": "));
    mfrc522[reader].PCD_DumpVersionToSerial();
  }
}

/*
   Main loop.
*/
void loop() {
  boolean changedValue = false;

  for (uint8_t reader = 0; reader < NR_OF_READERS; reader++) {
    mfrc522[reader].PCD_Init();
    String readRFID = "";
    // Look for new cards

    if (mfrc522[reader].PICC_IsNewCardPresent() && mfrc522[reader].PICC_ReadCardSerial()) {
      readRFID = dump_byte_array(mfrc522[reader].uid.uidByte, mfrc522[reader].uid.size, reader);

      // Halt PICC
      mfrc522[reader].PICC_HaltA();
      // Stop encryption on PCD
      mfrc522[reader].PCD_StopCrypto1();
    }

    if (readRFID != currentIDs[reader]) {
      changedValue = true;
      currentIDs[reader] = readRFID;
    }

  }
  if (changedValue) {
    for (uint8_t reader = 0; reader < NR_OF_READERS; reader++) {
      if (reader == 0) {
        Serial.print('A');
      }
      if (reader == 1) {
        Serial.print('B');
      }
      if (reader == 2) {
        Serial.print('C');
      }
      if (reader == 3) {
        Serial.print('D');
      }
      if (reader == 4) {
        Serial.print('E');
      }
      if (reader == 5) {
        Serial.print('F');
      }
      Serial.println(currentIDs[reader]);
    }
  }
  delay(500);
}


// Helper routine to return a byte array as hex values.

String dump_byte_array(byte * buffer, byte bufferSize, byte current) {
  String readRFID;
  for (byte j = 0; j < mfrc522[current].uid.size; j++)
  {
    readRFID.concat(String(mfrc522[current].uid.uidByte[j] < 0x10 ? "0" : ""));
    readRFID.concat(String(mfrc522[current].uid.uidByte[j], HEX));
  }
  readRFID.toUpperCase();
  return readRFID;
}
